@@ -1,4 +1,4 @@
-# Copyright (C) 2010  ParIT Worker Co-operative, Ltd <paritinfo@parit.ca>
+# Copyright (C) 2010-2011  ParIT Worker Co-operative, Ltd <paritinfo@parit.ca>
 #
 # This file is part of Bo-Keep.
 #
@@ -15,7 +15,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Author: Mark Jenkins <mark@parit.ca>
+# Authors: Mark Jenkins <mark@parit.ca>
+#          Samuel Pauls <samuel@parit.ca>
 
 # Python library
 from os.path import exists, abspath, dirname, join
@@ -25,11 +26,12 @@ import sys
 import transaction
 
 # Gtk
-from gtk import main_quit, ListStore, CellRendererText, AboutDialog
+from gtk import main_quit, ListStore, CellRendererText, AboutDialog, \
+    MessageDialog, MESSAGE_ERROR, BUTTONS_OK
 from gtk.glade import XML
 import gobject
 from gtk.gdk import pixbuf_new_from_file_at_size
-
+import gtk
 
 # Bo-Keep
 from state import \
@@ -40,6 +42,7 @@ from bokeep.book_transaction import \
 from bokeep.gui.gladesupport.glade_util import \
      load_glade_file_get_widgets_and_connect_signals
 from bokeep.config import \
+    get_bokeep_configuration, \
     get_bokeep_bookset, get_bokeep_config_paths, \
     first_config_file_in_list_to_exist_and_parse, \
     BoKeepConfigurationException, \
@@ -54,6 +57,24 @@ GUI_STATE_SUB_DB = 'gui_state'
 
 COMBO_SELECTION_NONE = -1
 
+def shell_startup(config_path, config, bookset, startup_callback):
+    shell_window = MainWindow(config_path, config, bookset, startup_callback)
+    gtk.main()
+
+def shell_startup_config_establish(config_path, e, *cbargs):
+    mainwindow = cbargs[0]
+    config_path = establish_bokeep_config(mainwindow, (config_path,), e)
+    try:
+        return ( (None, None)
+                 if config_path == None
+                 else config_path, get_bokeep_configuration(config_path) )
+    except BoKeepConfigurationFileException:
+        return None, None
+
+def shell_startup_bookset_fetch(config_path, config, e, *cbargs):
+    mainwindow = cbargs[0]
+    return establish_bokeep_db(mainwindow, config_path, config, e)
+
 def get_bo_keep_logo():
     import mainwindow as main_window_module
     return join( dirname( abspath(main_window_module.__file__)),
@@ -61,15 +82,20 @@ def get_bo_keep_logo():
 
 
 class MainWindow(object):
-    # Functions for window initialization 
+    # Functions for window initialization
 
-    def __init__(self, options, args):
+    def on_quit_activate(self, args):
+        self.application_shutdown()
+
+    def __init__(self, config_path, config, bookset, startup_callback):
         self.gui_built = False
         self.current_editor = None
-        self.cmdline_options = options
-        self.cmdline_args = args
+        self.config_path_and_config_set(config_path, config)
+        self.bookset_set(bookset)
+        
         self.build_gui()
         self.programmatic_transcombo_index = False
+        self.__startup_callback = startup_callback
 
         # program in an event that will only run once on startup
         # the startup_event_handler function will use
@@ -79,10 +105,21 @@ class MainWindow(object):
         # should we do an emit to ensure it happens, or be satisfyed
         # that it always happens in tests?
 
+    def config_path_and_config_set(self, config_path, config):
+        self.__config_path = config_path
+        self.__config = config        
+
+    def bookset_set(self, bookset):
+        self.bookset = bookset
+
     def flush_backend_of_book(self, book):
         book.get_backend_module().flush_backend()
         transaction.get().commit()
 
+    def close_backend_of_book(self, book):
+        book.get_backend_module().close()
+        transaction.get().commit()
+
     def application_shutdown(self):
         if hasattr(self, 'guistate'):
             self.guistate.do_action(CLOSE)
@@ -93,7 +130,7 @@ class MainWindow(object):
             self.bookset.close()
             # or, should I be only doing
             # self.bookset.close_primary_connection() instead..?
-        main_quit()       
+        main_quit()
 
     def startup_event_handler(self, *args):
         # this should only be programmed to run once
@@ -104,35 +141,13 @@ class MainWindow(object):
         delattr(self, 'startup_event_handler')
         assert(not self.gui_built)
 
-        config_paths = get_bokeep_config_paths(
-            self.cmdline_options.configfile)
-        try:
-            self.bookset = get_bokeep_bookset(config_paths[0])
-        except BoKeepConfigurationFileException, e:
-            config_path = establish_bokeep_config(
-                self.mainwindow, config_paths, e)
-            if config_path == None:
-                self.application_shutdown()
-                return
-            self.bookset = establish_bokeep_db(
-                self.mainwindow, config_path, None)
-            if self.bookset == None:
-                self.application_shutdown()
-                return
-        except BoKeepConfigurationDatabaseException, e:
-            config_path = first_config_file_in_list_to_exist_and_parse(
-                config_paths)
-            if config_path == None:
-                self.application_shutdown()
-                return
-            self.bookset = establish_bokeep_db(
-                self.mainwindow, config_path, e)
-            if self.bookset == None:
-                self.application_shutdown()
-                return
-
-        self.after_background_load()
-        assert(self.gui_built)
+        if self.__startup_callback(self.__config_path, self.__config,
+                                   self.config_path_and_config_set,
+                                   self.bookset_set, self.mainwindow ):
+            self.after_background_load()
+            assert(self.gui_built)
+        else:
+            self.application_shutdown()
 
     def build_gui(self):
         glade_file = get_main_window_glade_file()
@@ -146,7 +161,7 @@ class MainWindow(object):
         self.books_combobox.pack_start(cell, True)
         self.books_combobox.add_attribute(cell, 'text', 0)
         self.trans_type_model = ListStore(str, int, object)
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
         
     def after_background_load(self):
         self.guistate = (
@@ -198,7 +213,7 @@ class MainWindow(object):
         
         self.gui_built = True
 
-        self.refresh_trans_types_and_set_sensitivities()
+        self.refresh_trans_types_and_set_sensitivities_and_status()
 
     def closedown_for_config(self):
         self.gui_built = False
@@ -223,9 +238,9 @@ class MainWindow(object):
                 self.books_combobox.get_active()][2]
             )
 
-    def refresh_trans_types_and_set_sensitivities(self):
+    def refresh_trans_types_and_set_sensitivities_and_status(self):
         self.refresh_trans_types()
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
 
     def refresh_trans_types(self):
         book = self.guistate.get_book()
@@ -257,7 +272,7 @@ class MainWindow(object):
             self.set_transcombo_index(current_trans_type_index)
             self.reset_trans_view()
 
-    def set_transcombo_index(self, indx):        
+    def set_transcombo_index(self, indx):
         self.programmatic_transcombo_index = True
         self.trans_type_combo.set_active(indx)
         self.programmatic_transcombo_index = False      
@@ -271,15 +286,17 @@ class MainWindow(object):
             get_transaction_edit_interface_hook_from_code(currcode)
         self.clear_trans_view()
         trans_id = self.guistate.get_transaction_id()
+
         self.current_editor = editor_generator(
-            book.get_transaction(trans_id), trans_id, currmodule,
-            self.main_vbox, self.guistate.record_trans_dirty_in_backend)
+                book.get_transaction(trans_id), trans_id, currmodule,
+                self.transaction_viewport, self.guistate.record_trans_dirty_in_backend,
+                book)
 
     def clear_trans_view(self):
         if self.current_editor != None: 
             self.current_editor.detach()
 
-    def set_sensitivities(self):
+    def set_sensitivities_and_status(self):
         for (sensitive_widget, action_code) in \
                 ( (self.back_button, BACKWARD),
                   (self.forward_button, FORWARD),
@@ -296,6 +313,17 @@ class MainWindow(object):
                     sensitive_widget.set_sensitive(False)
 
         self.set_backend_error_indicator()
+        self.set_transid_label()
+
+    def set_transid_label(self):
+        if self.gui_built and not(self.guistate.get_book() == None):
+            last_trans_id = self.guistate.get_book().get_latest_transaction_id()
+            if last_trans_id != None:
+                self.transid_label.set_text(
+                "%s / %s" %
+                (self.guistate.get_transaction_id(), last_trans_id ))
+                return # avoid set_text("") below
+        self.transid_label.set_text("")
 
     def set_backend_error_indicator(self):
         # don't bother if the gui isn't built yet
@@ -308,11 +336,23 @@ class MainWindow(object):
             backend = book.get_backend_module()
             if backend.transaction_is_clean(trans_id):
                 self.backend_error_light.hide()
-                self.backend_error_msg_label.set_text("")
+                self.backend_error_msg_label.hide()
+                self.error_details_button.hide()
             else:
                 self.backend_error_light.show()
                 self.backend_error_msg_label.set_text(
                     backend.reason_transaction_is_dirty(trans_id) )
+                self.backend_error_msg_label.show()
+                self.error_details_button.show()
+
+    def on_error_details_button_clicked(self, *args):
+        md = MessageDialog(parent = self.mainwindow,
+                           type = MESSAGE_ERROR,
+                           buttons = BUTTONS_OK,
+                           message_format =
+                               self.backend_error_msg_label.get_text())
+        md.run()
+        md.destroy()
 
     # Functions for use to event handlers, not used during initialization
 
@@ -343,14 +383,14 @@ class MainWindow(object):
             return
 
         self.set_book_from_combo()
-        self.refresh_trans_types_and_set_sensitivities()
+        self.refresh_trans_types_and_set_sensitivities_and_status()
         
-    def new_button_clicked(self, *args):
+    def on_new_button_clicked(self, *args):
         self.guistate.do_action(NEW)
         self.set_trans_type_combo_to_current_and_reset_view()
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
 
-    def delete_button_clicked(self, *args):
+    def on_delete_button_clicked(self, *args):
         self.guistate.do_action(DELETE)
         book = self.guistate.get_book()
         if self.guistate.get_transaction_id() == None:
@@ -358,7 +398,7 @@ class MainWindow(object):
             self.clear_trans_view()
         else:
             self.set_trans_type_combo_to_current_and_reset_view()
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
         
     def trans_type_changed(self, *args):
         """Event handler for when the transaction type on a new transaction
@@ -378,17 +418,17 @@ class MainWindow(object):
             self.guistate.do_action(TYPE_CHANGE,
                                     self.trans_type_combo.get_active())
             self.reset_trans_view()
-            self.set_sensitivities()
+            self.set_sensitivities_and_status()
 
-    def forward_button_clicked(self, *args):
+    def on_forward_button_clicked(self, *args):
         self.guistate.do_action(FORWARD)
         self.set_trans_type_combo_to_current_and_reset_view()
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
     
-    def back_button_clicked(self, *args):
+    def on_back_button_clicked(self, *args):
         self.guistate.do_action(BACKWARD)
         self.set_trans_type_combo_to_current_and_reset_view()
-        self.set_sensitivities()
+        self.set_sensitivities_and_status()
 
     def on_remove(self, window, event):
         self.application_shutdown()
@@ -398,16 +438,12 @@ class MainWindow(object):
         self.closedown_for_config()
         self.bookset.close()
         assert( not self.gui_built )
-        # this probably isn't right, should actually retain config path
-        # from startup or even the config object, and adjust this api
-        # to take tht instead
-        config_paths = get_bokeep_config_paths(
-            self.cmdline_options.configfile)
+
         # major flaw right now is that we don't want this to
         # re-open the same DB at the end, and we need to do something
         # the return value and seting bookset
         self.bookset = \
-            establish_bokeep_db(self.mainwindow, config_paths[0], None)
+            establish_bokeep_db(self.mainwindow, self.__config_path, self.__config, None)
 
         # if there's uncommited stuff, we need to ditch it because
         # the above function killed off and re-opened the db connecion
@@ -443,9 +479,11 @@ class MainWindow(object):
             return
         currmodule = self.trans_type_combo.get_model().get_value(
             currindex,2)
+
         currmodule.run_configuration_interface(
             self.mainwindow, self.guistate.get_book().get_backend_module(
-                ).backend_account_dialog)
+                ).backend_account_dialog,
+            self.guistate.get_book())
         # hmm, this doesn't seem to be getting it done
         self.clear_trans_view()
         self.reset_trans_view()
@@ -457,7 +495,7 @@ class MainWindow(object):
         ab.set_transient_for(self.mainwindow)
         ab.set_modal(True)
         ab.set_name("Bo-Keep")
-        ab.set_version("1.0")
+        ab.set_version("1.1.1")
         ab.set_copyright("ParIT Worker Co-operative, Ltd. 2006-2011")
         ab.set_comments(
             """Bo-Keep helps you keep your books so you don't get lost.
@@ -481,7 +519,10 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """)
         ab.set_authors(("Mark Jenkins <mark@parit.ca>",
-                        "Jamie Campbell <jamie@parit.ca>"))
+                        "Jamie Campbell <jamie@parit.ca>",
+                        "Samuel Pauls <samuel@parit.ca>",
+                        "Andrew Orr <andrew@andreworr.ca>",
+                        "Sara Arenson <sara_arenson@yahoo.ca>",))
         ab.set_artists(("David Henry <work@davidhenry.ca>",))
         ab.set_program_name("Bo-Keep")
         ab.set_logo( pixbuf_new_from_file_at_size(
@@ -494,3 +535,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
             return
         self.flush_backend_of_book(self.guistate.get_book())
         self.set_backend_error_indicator()
+
+    def on_backend_close_request(self, *args):
+        book = self.guistate.get_book()
+        if book == None:
+            return
+        self.flush_backend_of_book(book)
+        self.close_backend_of_book(book)
+        self.set_backend_error_indicator()
