@@ -1,4 +1,5 @@
-# Copyright (C) 2010  ParIT Worker Co-operative, Ltd <paritinfo@parit.ca>
+# Copyright (C) 2010-2010  ParIT Worker Co-operative, Ltd <paritinfo@parit.ca>
+# Copyright (C) 2011 SkullSpace Winnipeg Inc. <andrew@andreworr.ca>
 #
 # This file is part of Bo-Keep.
 #
@@ -16,16 +17,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 # Author: Mark Jenkins <mark@parit.ca>
+# Author: Andrew Orr <andrew@andreworr.ca>
 
 # python standard library
 from threading import Thread, Condition, Event
 from os.path import abspath, dirname, join, exists, basename
+from datetime import date, timedelta
+from zlib import adler32
+from decimal import Decimal, ROUND_DOWN, ROUND_HALF_UP
+from itertools import chain
 
 # ZODB
 import transaction
 from persistent import Persistent
 
-# simplifies common ussage of transaction.get().commit()
+# simplifies common usage of transaction.get().commit()
 def ends_with_commit(dec_function):
     def ret_func(*args, **kargs):
         return_value = dec_function(*args, **kargs)
@@ -43,7 +49,6 @@ def none_args_become_dict(function):
         return function( *new_args, **kargs )
     return new_function
 
-
 def attribute_or_blank(obj, attr):
     if hasattr(obj, attr):
         return getattr(obj, attr)
@@ -450,7 +455,18 @@ class StateMachineMinChangeDataStore(object):
     def get_value(self, key):
         return self.__values[key]
 
+def enhance_syspath_for_module_load(path, position=0):
+    import sys
+    directory = dirname(abspath(path))
+    filename = basename(path)
+    sys.path.insert(position, directory)
+
+def drop_syspath_enhancement(position=0):
+    import sys
+    sys.path.pop(position) # cleanup
+
 def get_module_for_file_path(path):
+    PATH_MOD_POSITION = 0
     # The're got to be a nicer way to load code from a file...
     import sys
     PYTHON_EXTENSION = ".py"
@@ -459,17 +475,21 @@ def get_module_for_file_path(path):
     directory = dirname(abspath(path))
     filename = basename(path)
     assert( filename.endswith(PYTHON_EXTENSION) )
-    modulename = filename[:-3] # everything but last three characterrs
-    PATH_MOD_POSITION = 0
     sys.path.insert(PATH_MOD_POSITION, directory)
+    modulename = filename[:-3] # everything but last three characterrs
     try:
         return_value = __import__(modulename,  globals(), locals(), [""])
     except ImportError:
         return_value = None
     finally:
-        sys.path.pop(PATH_MOD_POSITION) # cleanup
+        drop_syspath_enhancement(PATH_MOD_POSITION)
     return return_value
 
+def reload_module_at_filepath(module, path):
+    enhance_syspath_for_module_load(path)
+    reload(module)
+    drop_syspath_enhancement()
+
 class PluginSet(Persistent):
     def __init__(self):
         self.enabled_plugins = {}
@@ -515,3 +535,84 @@ class PluginSet(Persistent):
 def get_file_in_same_dir_as_module(module, filename): 
     return join( dirname( abspath( module.__file__ ) ),
                  filename )
+
+def first_of(a_date):
+    return date(a_date.year, a_date.month, 1)
+
+def last_of_month(a_date):
+    return month_delta(first_of(a_date), 1) - timedelta(days=1)
+
+def month_delta(current_date, months=1):
+    """Always call with a day that is between 1 and 28, other dates can
+    be invalid for shifting by month
+    """
+    assert( 1<= current_date.day <=28 )
+    new_month = current_date.month + months
+    new_year = current_date.year
+    if new_month > 12 or new_month >= 0:
+        # FIXME
+        # this could use some heavy testing...
+        new_year+= (new_month-1) / 12
+        new_month = ((new_month-1) % 12) + 1
+    return date(new_year, new_month, current_date.day)
+
+
+def start_of_year(a_date):
+    return date(a_date.year, 1, 1)
+
+def adler32_of_file(file_path):
+    """Very fast for file comparison, but a super poor choice if
+    used for cyrptographic uses -- and you thought md5 sucked...
+    """
+    f = file(file_path)
+    return_value = adler32(''.join(f)) & 0xffffffff
+    f.close()
+    return return_value
+
+def get_and_establish_attribute(obj, attr, default_cls, *args, **kargs):
+    return_attr = getattr(obj, attr,
+                          default_cls(*args, **kargs) )
+    setattr(obj, attr, return_attr )
+    return return_attr
+
+def null_function(*args, **kargs): pass
+
+# useful with Decimal.quantize
+TWOPLACES = Decimal('0.01')
+ZEROPLACES = Decimal('1.')
+
+def decimal_round_two_place_using_third_digit(decimal_value):
+    # we want these results eh?
+    # >>> Decimal('0.025').quantize(Decimal('0.00'), rounding=ROUND_HALF_UP)
+    # Decimal("0.03")
+    # >>> Decimal('0.015').quantize(Decimal('0.00'), rounding=ROUND_HALF_UP)
+    # Decimal("0.02")
+    # >>> Decimal('-0.015').quantize(Decimal('0.00'), rounding=ROUND_HALF_UP)
+    # Decimal("-0.02")
+    # >>> Decimal('-0.025').quantize(Decimal('0.00'), rounding=ROUND_HALF_UP)
+    # Decimal("-0.03")
+    #
+    # and this
+    # >>> Decimal('0.02499999999999999').quantize(Decimal('0.00'),
+    #  rounding=ROUND_HALF_UP)
+    # Decimal("0.02")
+    #
+    # Not this
+    # >>> Decimal('0.02499999999999999').quantize(Decimal('0.00'),
+    # rounding=ROUND_UP)
+    # Decimal("0.03")
+
+    return decimal_value.quantize(TWOPLACES, ROUND_HALF_UP)
+
+def decimal_truncate_two_places(decimal_value):
+    """ round something like 0.019 to 0.01 and -0.019 to -0.01
+    """
+    # note that using ROUND_FLOOR wouldn't achieve the desired results
+    # with negative numbers
+    return decimal_value.quantize(TWOPLACES, rounding=ROUND_DOWN)
+
+def tup_chain(*args):
+    return tuple(chain(*args))
+
+def tup_multi_append(original_tuple, *args):
+    return tup_chain(original_tuple, args)
