@@ -53,10 +53,16 @@ SQLITE3 = XML
 ASSETS_ACCOUNT = 'Assets'
 BANK_ACCOUNT = 'Bank'
 PETTY_CASH_ACCOUNT = 'Petty Cash'
+INCOME_ACCOUNT = 'Income'
+LIABILITIES_ACCOUNT = 'Liabilities'
+UNEARNED_REVENUE_ACCOUNT = 'Unearned Revenue'
 
-ASSETS_FULL_SPEC = (ASSETS_ACCOUNT)
+ASSETS_FULL_SPEC = (ASSETS_ACCOUNT,)
 BANK_FULL_SPEC = (ASSETS_ACCOUNT, BANK_ACCOUNT)
 PETTY_CASH_FULL_SPEC = (ASSETS_ACCOUNT, PETTY_CASH_ACCOUNT)
+INCOME_FULL_SPEC = (INCOME_ACCOUNT,)
+LIABILITIES_FULL_SPEC = (LIABILITIES_ACCOUNT,)
+UNEARNED_REVENUE_FULL_SPEC = (LIABILITIES_ACCOUNT, UNEARNED_REVENUE_ACCOUNT)
 
 class TestTransaction(Transaction):
     def __init__(self, value1, account1, value2, account2):
@@ -72,10 +78,8 @@ class TestTransaction(Transaction):
     def get_financial_transactions(self):
        return [self.fin_trans]
 
-class GnuCashBasicSetup(TestCase):
+class GnuCashFileSetup(TestCase):
     def setUp(self):
-        from bokeep.backend_plugins.gnucash_backend import \
-            GnuCash
         from gnucash import Account, GnuCashBackendException
         from gnucash.gnucash_core_c import \
             ACCT_TYPE_ASSET, ERR_FILEIO_BACKUP_ERROR 
@@ -102,6 +106,10 @@ class GnuCashBasicSetup(TestCase):
             return return_value
         
         assets = create_new_account(ASSETS_ACCOUNT, root)
+        liabilities = create_new_account(LIABILITIES_ACCOUNT, root)
+        income = create_new_account(INCOME_ACCOUNT, root)
+        unearned_revenue = create_new_account(UNEARNED_REVENUE_ACCOUNT,
+                                              liabilities)
         bank = create_new_account(BANK_ACCOUNT, assets)
         petty_cash = create_new_account(PETTY_CASH_ACCOUNT, assets)
         try:
@@ -115,15 +123,7 @@ class GnuCashBasicSetup(TestCase):
                 raise e
         self.gnucash_session_termination(s)
 
-        self.backend_module = GnuCash()
-        self.assertFalse(self.backend_module.can_write())
-        self.backend_module.setattr(
-            'gnucash_file', self.get_gnucash_file_name_with_protocol() )
-        self.assert_(self.backend_module.can_write())
-
     def tearDown(self):
-        self.backend_module.close()
-        self.assertFalse(self.backend_module.can_write())
         for file_name in glob(self.gnucash_file_name + '*'):
             remove(file_name)
 
@@ -142,6 +142,13 @@ class GnuCashBasicSetup(TestCase):
     def get_protocol_full(self):
         return self.get_protocol() + "://"
 
+
+    def do_test_for_sub_account(self, parent, sub_name):
+        sub = parent.lookup_by_name(sub_name)
+        self.assertNotEquals(sub.get_instance(), None)
+        self.assertEquals(sub.GetName(), sub_name)
+        return sub
+
     def check_account_tree_is_present(self, session_provided=None):
         if session_provided == None:
             self.backend_module.close()
@@ -151,12 +158,6 @@ class GnuCashBasicSetup(TestCase):
             book = s.book
             root = book.get_root_account()
 
-        def test_for_sub_account(parent, sub_name):
-            sub = parent.lookup_by_name(sub_name)
-            self.assertNotEquals(sub.get_instance(), None)
-            self.assertEquals(sub.GetName(), sub_name)
-            return sub
-
         self.acquire_test_accounts_from_root(root)
         
         if session_provided == None:
@@ -170,14 +171,9 @@ class GnuCashBasicSetup(TestCase):
         return (s, book, root)
 
     def acquire_test_accounts_from_root(self, root):
-        def test_for_sub_account(parent, sub_name):
-            sub = parent.lookup_by_name(sub_name)
-            self.assertNotEquals(sub.get_instance(), None)
-            self.assertEquals(sub.GetName(), sub_name)
-            return sub
-        assets = test_for_sub_account(root, ASSETS_ACCOUNT)
-        bank = test_for_sub_account(assets, BANK_ACCOUNT)
-        petty_cash = test_for_sub_account(assets, PETTY_CASH_ACCOUNT)
+        assets = self.do_test_for_sub_account(root, ASSETS_ACCOUNT)
+        bank = self.do_test_for_sub_account(assets, BANK_ACCOUNT)
+        petty_cash = self.do_test_for_sub_account(assets, PETTY_CASH_ACCOUNT)
         return (assets, bank, petty_cash)
 
     def acquire_gnucash_session_book_root_and_accounts(self):
@@ -192,6 +188,37 @@ class GnuCashBasicSetup(TestCase):
         s.end()
         s.destroy()
 
+class GnuCashBasicSetup(GnuCashFileSetup):
+    def setUp(self):
+        from bokeep.backend_plugins.gnucash_backend import \
+            GnuCash
+
+        GnuCashFileSetup.setUp(self)
+        self.backend_module = GnuCash()
+        self.assertFalse(self.backend_module.can_write())
+        
+        # this early flush is something that bokeep.book.Book does
+        # right after backend plugin loading
+        #
+        # the absense of this closer emulation made this entire test file
+        # pretty useless in detecting bug #33616 
+        self.backend_module.flush_backend()
+
+        # the above flush should have failed due to no gnucash file
+        # having been established yet; back when we were pursuing
+        # bug #33616 this assert would fail... the above flush
+        # would work despite no actual file name having been defined yet
+        self.assertFalse(self.backend_module.can_write())
+
+        # underneath this calls flush_backend() again
+        self.backend_module.setattr(
+            'gnucash_file', self.get_gnucash_file_name_with_protocol() )
+        self.assert_(self.backend_module.can_write())
+
+    def tearDown(self):
+        self.backend_module.close()
+        self.assertFalse(self.backend_module.can_write())
+        GnuCashFileSetup.tearDown(self)
 
 class GetProtocolXML(object):
     def get_protocol(self):
@@ -220,13 +247,112 @@ class GnuCashBasicTest(GnuCashBasicSetup):
         self.assert_(self.backend_module.can_write() )
         self.do_close_and_tree_check()
 
+    def test_imbalance(self):
+        test_trans = TestTransaction(
+            Decimal(1), BANK_FULL_SPEC,
+            Decimal(-2), PETTY_CASH_FULL_SPEC )
+        test_trans.set_currency(self.get_currency())
+        front_end_id = 1
+        self.backend_module.mark_transaction_dirty(
+            front_end_id, test_trans)        
+        self.backend_module.flush_backend()
+        self.assertFalse(self.backend_module.transaction_is_clean(
+                front_end_id) )
+        self.assert_(
+            self.backend_module.reason_transaction_is_dirty(
+                front_end_id).endswith(
+                "transaction doesn't balance"))
+
+    def test_double_close(self):
+        self.backend_module.close()
+        self.backend_module.close()
+
+    def test_account_creation_when_not_there(self):
+        TEST_NEW_ACCOUNT = "test created account"
+        TEST_NEW_ACCOUNT_FULL_SPEC = (ASSETS_ACCOUNT, "test created account")
+        test_trans = TestTransaction(
+            Decimal(1), TEST_NEW_ACCOUNT_FULL_SPEC,
+            Decimal(-1), PETTY_CASH_FULL_SPEC )
+        test_trans.fin_trans.lines[0].create_account_if_missing = True
+        test_trans.set_currency(self.get_currency())
+        front_end_id = 1
+        self.backend_module.mark_transaction_dirty(
+            front_end_id, test_trans)
+        self.backend_module.flush_backend()
+        if not self.backend_module.transaction_is_clean(front_end_id):
+            self.assertEquals(
+                None, self.backend_module.reason_transaction_is_dirty(
+                    front_end_id) )
+        self.assert_(self.backend_module.transaction_is_clean(front_end_id))
+        self.backend_module.close()
+        self.assertFalse(self.backend_module.can_write() )
+        (s, book, root, accounts) = \
+            self.acquire_gnucash_session_book_root_and_accounts()
+        assets, bank, petty_cash = accounts[:3]
+        test_new_account = self.do_test_for_sub_account(
+            assets, BANK_ACCOUNT)
+        test_new_account = self.do_test_for_sub_account(
+            assets, TEST_NEW_ACCOUNT)
+        # next thing we should do is test that the transaction is
+        # actually there in the sub account, not just that the
+        # account was created
+
 class GnuCashBasicTestXML(GetProtocolXML, GnuCashBasicTest): pass
 
 class GnuCashBasicTestUSD(GetCurrencyUSD, GnuCashBasicTest): pass
 
-class GnuCashStartsWithMarkSetup(GnuCashBasicSetup):
+class GnuCashAlternativeBasicSetup(GnuCashBasicSetup):
+    """See inline comment in setUp() for why this TestCase and children
+    exist
+    """
     def setUp(self):
+        from bokeep.backend_plugins.gnucash_backend import \
+            GnuCash
+
+        # we don't call GnuCashBasicSetup.setUp because this implementation
+        # of setUp is overriding its behavior
+        GnuCashFileSetup.setUp(self)
+        self.backend_module = GnuCash()
+        self.assertFalse(self.backend_module.can_write())
+        
+        # what makes these tests "alternative" is that we don't
+        # do backend.module.flush_backend() at this point like
+        # bokeep.book.Book does. No requirement at all to do things that
+        # way, and seeing how GnuCashBasicSetup had previously done
+        # things that way, this "alternative" test setup was
+        # put in place so we could keep a bunch of tests done that way
+        # even after GnuCashBasicSetup was altered to behave more like
+        # how bokeep.book.Book and users of it behave.
+
+        # back when we were pursuing bug #33616 this would only fail if
+        # self.backend_module.flush_backend was called first
+        # but with that not being the sequence above we've never seen it
+        # fail
+        self.assertFalse(self.backend_module.can_write())
+
+        # underneath this calls flush_backend() (for the first time
+        # here in the alternative test set)
+        self.backend_module.setattr(
+            'gnucash_file', self.get_gnucash_file_name_with_protocol() )
+        self.assert_(self.backend_module.can_write())
+
+class GnuCashAlternativeBasicTests(
+    GnuCashAlternativeBasicSetup, GnuCashBasicTest): pass
+
+class GnuCashAlternativeBasicTestXML(
+    GetProtocolXML, GnuCashAlternativeBasicTests): pass
+
+class GnuCashAlternativeBasicTestUSD(
+    GetCurrencyUSD, GnuCashAlternativeBasicTests): pass
+
+class GnuCashStartsWithMarkSetup(GnuCashBasicSetup):
+    def designated_superclass_setUp(self):
         GnuCashBasicSetup.setUp(self)
+        
+    def setUp(self):
+        # method resolution order kludge, overridden by subclasses
+        self.designated_superclass_setUp()
+
         self.test_trans = TestTransaction(Decimal(1), BANK_FULL_SPEC,
                                           Decimal(-1), PETTY_CASH_FULL_SPEC )
         self.test_trans.set_currency(self.get_currency())
@@ -243,10 +369,8 @@ class GnuCashStartsWithMarkSetup(GnuCashBasicSetup):
         assets, bank, petty_cash = accounts[:3]
 
         return_value = False
-        bank_splits = [Split(instance=split_inst)
-                       for split_inst in bank.GetSplitList() ]
-        petty_cash_splits = [Split(instance=split_inst)
-                             for split_inst in petty_cash.GetSplitList() ]
+        bank_splits = bank.GetSplitList()
+        petty_cash_splits = petty_cash.GetSplitList()
         ONE = GncNumeric(1, 1)
         NEG_ONE = GncNumeric(-1, 1)
 
@@ -266,6 +390,13 @@ class GnuCashStartsWithMarkSetup(GnuCashBasicSetup):
 
         return return_value
 
+class GnuCashStartsWithMarkAlternativeSetup(
+    GnuCashStartsWithMarkSetup, GnuCashAlternativeBasicSetup):
+
+    def designated_superclass_setUp(self):
+        # method resolution order kludge
+        GnuCashAlternativeBasicSetup.setUp(self)
+
 class GnuCashStartsWithMarkTests(GnuCashStartsWithMarkSetup):   
     def test_simple_flush(self):
         self.backend_module.flush_backend()
@@ -338,10 +469,8 @@ class GnuCashStartsWithMarkTests(GnuCashStartsWithMarkSetup):
         assets, bank, petty_cash = accounts[:3]
 
         return_value = False
-        bank_splits = [Split(instance=split_inst)
-                       for split_inst in bank.GetSplitList() ]
-        petty_cash_splits = [Split(instance=split_inst)
-                             for split_inst in petty_cash.GetSplitList() ]
+        bank_splits = bank.GetSplitList()
+        petty_cash_splits = petty_cash.GetSplitList()
         self.assertEquals(len(bank_splits), 0)
         self.assertEquals(len(petty_cash_splits), 0)
         self.gnucash_session_termination(s)
@@ -381,6 +510,18 @@ class GnuCashStartsWithMarkTestsUSD(
     GetCurrencyUSD, GnuCashStartsWithMarkTests):
     pass
 
+class GnuCashStartsWithMarkAlternativeTests(
+    GnuCashStartsWithMarkAlternativeSetup, GnuCashStartsWithMarkTests):
+    pass
+
+class GnuCashStartsWithMarkTestsAlternativeXML(
+    GetProtocolXML, GnuCashStartsWithMarkAlternativeTests):
+    pass
+
+class GnuCashStartsWithMarkTestsAlternativeUSD(
+    GetCurrencyUSD, GnuCashStartsWithMarkAlternativeTests):
+    pass
+
 if __name__ == "__main__":
     main()
 
